package net.awired.ajsl.service.implementation.abstracts;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;
import net.awired.ajsl.persistence.dao.interfaces.abstracts.AbstractDAO;
import net.awired.ajsl.persistence.entity.interfaces.abstracts.IdEntity;
import net.awired.ajsl.service.interfaces.AbstractService;
import org.hibernate.LockMode;
import org.springframework.dao.DataAccessException;

public abstract class AbstractServiceImpl<ENTITY extends IdEntity<KEY_TYPE>, KEY_TYPE extends Serializable, DAO extends AbstractDAO<ENTITY, KEY_TYPE>>
        implements AbstractService<ENTITY, KEY_TYPE> {

    protected DAO dao;

    protected abstract void setDao(DAO dao);

    @Override
    public void persist(ENTITY transientInstance) throws DataAccessException {
        dao.persist(transientInstance);
    }

    @Override
    public List<ENTITY> findAll() throws DataAccessException {
        return dao.findAll();
    }

    @Override
    public ENTITY findById(KEY_TYPE id) throws DataAccessException {
        return dao.findById(id);
    }

    @Override
    public List<ENTITY> findByIdList(Collection<KEY_TYPE> idList) throws DataAccessException {
        return dao.findByIdList(idList);
    }

    @Override
    public List<ENTITY> loadAll() throws DataAccessException {
        return dao.loadAll();
    }

    @Override
    public void deleteAll() throws DataAccessException {
        dao.deleteAll();
    }

    @Override
    public void removeAll() throws DataAccessException {
        dao.removeAll();
    }

    @Override
    public void save(ENTITY transientInstance) throws DataAccessException {
        dao.save(transientInstance);
    }

    @Override
    public void update(ENTITY transientInstance) throws DataAccessException {
        dao.update(transientInstance);
    }

    @Override
    public ENTITY merge(ENTITY detachedEntity) throws DataAccessException {
        return dao.merge(detachedEntity);
    }

    public void delete(ENTITY entity) throws DataAccessException {
        dao.delete(entity);
    }

    //TODO do we ahve to delete this ?
    public void delete(ENTITY entity, LockMode mode) throws DataAccessException {
        dao.delete(entity, mode);
    }

}
